import { Injectable } from "@angular/core";
import io from "socket.io-client";
import { Events } from 'ionic-angular';
import { Observable } from "rxjs/Observable";

import { Config } from '../config';
import { Device } from '../device';

import { iChannel } from '../models/channel';
import { iDevice } from '../models/device';

@Injectable()
export class Socket {

  channel: iChannel;
  device: iDevice;
  domain: string;
  key: string;
  deviceProvider: Device;
  socket: io.Socket;
  isConnected: boolean;
  private eventRegister;


  constructor(deviceProvider: Device, private events: Events, private config: Config) {
    this.deviceProvider = deviceProvider;
    this.eventRegister = [];
  }


  public connected(): Boolean {
    return this.socket.connected;
  }

  public disconnect() {
    this.socket.disconnect();
  }

  public getSocket() {
    return this.socket;
  }



  public connect() {
    return new Promise((resolve, reject) => {
      let that = this;

      Promise.all([
        this.deviceProvider.getDevice(),
        this.config.initialize()
      ]).then((config) => {

        this.domain = this.config.get('domain', false);
        this.channel = this.config.get('core', false);
        this.key = this.config.get('key', false);
        this.device = config[0] as any;

        let socketOptions = this.getSocketOptions();
        let socketEndpoint = this.channel.protocol + '://' + this.channel.host + ':' + this.channel.port;

        console.debug("atajo.socket : SOCKET CONNECTING WITH KEY " + this.key + " TO " + socketEndpoint + " - ", socketOptions);

        if (this.socket) {
          this.socket.destroy();
          delete this.socket;
        }

        this.socket = io(socketEndpoint, socketOptions);

        this.addEvent('disconnect', data => {
          console.warn("SOCKET DISCONNECTED : ", data);
        });

        this.addEvent('connect', data => {
          that.socket.emit('provider:status', { domain: this.domain });
        });

        this.addEvent('transaction', data => { });

        this.addEvent('provider:connect', data => {
          console.debug("atajo.socket :PROVIDER:CONNECT", data);
        });

        this.addEvent('provider:disconnect', data => {
          console.debug("atajo.socket :PROVIDER:DISCONNECT", data);
        });

        this.addEvent('provider:status', data => {
          console.debug("atajo.socket :PROVIDER:STATUS", data);
        })

        this.addEvent('client:rx', data => {
          console.debug("atajo.socket :CLIENT:RX", data);
        });

        resolve();

      }, () => {
        reject();
      });
    });
  }


  public transaction(transaction) {

    let tx = {
      version: transaction.version,
      lambda: transaction.lambda,
      domain: this.config.get('domain', false),
      environment: this.config.get('environment', false),
      payload: transaction.payload,
      latency: transaction.latency
    };

    console.debug("atajo.socket : CLIENT:TX", tx);

    this.socket.emit('client:tx', tx);

  }


  public addEvent(eventName: string, execute) {

    console.debug("atajo.socket :ADDING EVENT " + eventName + " TO SOCKETS");
    if (!this.socket) {
      console.error("SOCKET NOT INITIALIZED YET - CANNOT ADD EVENT");
      return;
    }

    this.eventRegister.push(eventName);
    this.socket.on(eventName, data => {
      console.debug("atajo.socket :GOT SOCKET EVENT : " + eventName + " => PUBLISHING TO " + 'socket:' + eventName);
      data = execute(data) || data;
      this.events.publish('socket:' + eventName, data);
    });

  }

  public removeEvent(eventName: string) {
    this.socket.removeListener(eventName);
    var index = this.eventRegister.indexOf(eventName, 0);
    if (index > -1) {
      this.eventRegister.splice(index, 1);
    }
    return this.eventRegister;
  }

  public listEvents() {

    return this.eventRegister;

  }


  private getSocketOptions() {

    let options = this.config.get('socket.options', {});
    options.query = {
      uuid: this.device.uuid,
      key: this.key,
      domain: this.domain,
      device: JSON.stringify(this.device)
    }
    return options;

  }

}
